import logging
import socket
import threading
import asynchat
import asyncore
import struct

# logging.basicConfig(level=logging.DEBUG)
log = logging.getLogger(__name__)

UDP_IP = "127.0.0.1"
UDP_PORT = 4321
TIMEOUT = 0.1


class UdpHandler(asynchat.async_chat):
    """
    Handler of UDP pipe from AVR simulator. It provides basic network
    facilities. It shall be subclassed to implement specific behavior.
    """
    def __init__(self, ip=UDP_IP, port=UDP_PORT, timeout=TIMEOUT):
        asynchat.async_chat.__init__(self)
        self.set_terminator(1)
        self.create_socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.socket.settimeout(timeout)
        self.socket.setblocking(False)
        self.ibuffer = ''
        self.ip = ip
        self.port = port

    def start(self):
        self.socket.sendto('', (self.ip, self.port))
        self._thread = threading.Thread(target=lambda:
                                        asyncore.loop(timeout=1),
                                        name='UdpHandlerThread')
        self._thread.start()

    def collect_incoming_data(self, data):
        self.ibuffer += data

    def terminate(self):
        self.close()
        self._thread.join()


class UdpReader(UdpHandler):
    """ This handler does not answer anything. It accumulates data
    which it receives, i.e. does the same as the original one.
    """
    def found_terminator(self):
        pass

    def read(self):
        x = self.ibuffer
        self.ibuffer = ''
        return ''.join(x)


class UdpRepeater(UdpHandler):
    """ This handler repeats everything it receives back. """
    def __init__(self, ip=UDP_IP, port=UDP_PORT, timeout=TIMEOUT):
        super(UdpRepeater, self).__init__(ip, port, timeout)
        self.set_terminator(1)

    def found_terminator(self):
        resp = self.ibuffer
        self.ibuffer = ''
        self.socket.sendto(resp, (self.ip, self.port))


class HypotheticDevice(UdpHandler):
    """
    This class is example how to implement a set of UART connected devices
    communicating via simple protocol: receives address byte and command and
    response with 2 bytes of data.
    """
    def __init__(self, ip=UDP_IP, port=UDP_PORT, timeout=TIMEOUT):
        super(UdpRepeater, self).__init__(ip, port, timeout)
        self.set_terminator(1)
        self.addr = None

    def found_terminator(self):
        if self.addr == None:
            self.addr = struct.unpack('B', self.ibuffer)
        else:
            command = struct.unpack('B', self.ibuffer)
            response = self.reactOnCommand(self.addr, command)
            self.send(response)
            self.addr = None



''' unit test example'''

from pysimavr.sim import ArduinoSim


def test_atmega88():
    mcu = 'atmega88'
    snippet = 'Serial.print("hi");'

    output = ArduinoSim(snippet=snippet, mcu=mcu, timespan=0.01).get_serial()
    assert output == 'hi'



from entrypoint2 import entrypoint
from pysimavr.sim import ArduinoSim


@entrypoint
def run_sim(vcdfile='delay.vcd'):
    snippet = '''
        Serial.println("start");
        pinMode(0, OUTPUT);
        digitalWrite(0, HIGH);
        delay(100);
        digitalWrite(0, LOW);
        delay(100);
        digitalWrite(0, HIGH);
        delay(100);
        digitalWrite(0, LOW);
        delay(100);
        Serial.println("end");
    '''
    sim = ArduinoSim(snippet=snippet, vcd=vcdfile, timespan=0.5)
    sim.run()



from setuptools.extension import Extension
from paver.easy import *
from paver.setuputils import setup
from setuptools import find_packages

import paver.doctools
import paver.misctasks
from paved import *
from paved.dist import *
from paved.util import *
from paved.docs import *
from paved.pycheck import *
from paved.pkg import *
from sphinxcontrib import paverutils

# get info from setup.py
setup_py = ''.join(
    [x for x in path('setup.py').lines() if 'setuptools' not in x])
exec(setup_py)


options(
    sphinx=Bunch(
        docroot='docs',
        builddir="_build",
    ),
    pdf=Bunch(
        builddir='_build',
        builder='latex',
    ),
)

options.paved.clean.rmdirs += ['.tox',
                               'dist',
                               'build',
                               ]
options.paved.clean.patterns += ['*.pickle',
                                 '*.doctree',
                                 '*.gz',
                                 'nosetests.xml',
                                 'sloccount.sc',
                                 '*.pdf', '*.tex',
                                 '*.png',
                                 '*.o', '*.os',  # object files
                                 '*.so',  # libs
                                 '*_wrap.c',  # generated by swig
                                 '*.vcd',
                                 '*.zip',
                                 'distribute_setup.py',
                                 ]

options.paved.dist.manifest.include.remove('distribute_setup.py')
options.paved.dist.manifest.include.remove('paver-minilib.zip')
options.paved.dist.manifest.recursive_include.add('pysimavr *.h')
options.paved.dist.manifest.include.add('requirements.txt')


@task
@needs(
    #           'clean',
    'scons',
    'sloccount',
    'html',
    'pdf',
    'sdist',
    'nose', 'tox',
)
def alltest():
    'all tasks to check'
    pass


@task
@needs('sphinxcontrib.paverutils.html')
def html():
    pass


@task
@needs('sphinxcontrib.paverutils.pdf')
def pdf():
    'Generate PDF output and copy into html directory'
    fpdf = list(path('docs/_build/latex').walkfiles('*.pdf'))[0]
    d = path('docs/_build/html')
    d.makedirs()
    fpdf.copy(d)


@task
#    @needs('paved.paved.clean') # this does not work! strange
def clean(options, info):
    paved.clean(options, info)
    info("Cleaning swig files")
    d = path('pysimavr') / 'swig'
    for x in d.walkfiles('*.py'):
        if path(x).stripext() + '.i' in d.files('*.i'):
            x.remove()


@task
def scons():
    sh('scons', cwd='pysimavr/swig')


@task
@needs(
#       'bdist_egg', 
       'sdist', 
       'distutils.command.upload',
       )
def upload():
    pass


@task
def tox():
    '''Run tox.'''
    sh('tox')


@task
@needs('manifest', 'setuptools.command.sdist')
def sdist():
    """Overrides sdist to make sure that our MANIFEST.in is generated.
    """
    pass


@task
@needs('scons', 
       'paved.pycheck.nose')
def nose():
    """Overrides nose to make sure that scons is run.
    """
    pass


@task
@needs('scons')
def build():
    ''




from path import path
import logging
import sphinx
import sys
from setuptools import find_packages


def read_project_version(py=None, where='.', exclude=['bootstrap', 'pavement', 'doc', 'docs', 'test', 'tests', ]):
    if not py:
        py = path(where) / find_packages(where=where, exclude=exclude)[0]
    py = path(py)
    if py.isdir():
        py = py / '__init__.py'
    __version__ = None
    for line in py.lines():
        if '__version__' in line:
            exec line
            break
    return __version__


release = read_project_version(where='..')
project = 'pysimavr'
author = 'ponty'
copyright = '2011, ponty'

# logging.basicConfig(level=logging.DEBUG)

# Extension
extensions = [
    # -*-Extensions: -*-
    'sphinx.ext.autodoc',
    'sphinxcontrib.programoutput',
    'sphinxcontrib.programscreenshot',
    'sphinxcontrib.gtkwave',
    #     'sphinx.ext.graphviz',
    'sphinxcontrib.autorun',
    #'sphinx.ext.autosummary',
    #     'sphinx.ext.intersphinx',
]
# intersphinx_mapping = {'http://docs.python.org/': None}

# Source
master_doc = 'index'
templates_path = ['_templates']
source_suffix = '.rst'
exclude_trees = []
pygments_style = 'sphinx'

# html build settings
html_theme = 'default'
html_static_path = ['_static']

# htmlhelp settings
htmlhelp_basename = '%sdoc' % project

# latex build settings
latex_documents = [
    ('index', '%s.tex' % project, u'%s Documentation' % project,
     author, 'manual'),
]

# remove blank pages from pdf
# http://groups.google.com/group/sphinx-
# dev/browse_thread/thread/92e19267d095412d/d60dcba483c6b13d
latex_font_size = '10pt,oneside'

latex_elements = dict(
    papersize='a4paper',
)



from pysimavr.swig.simavr import use_mem_logger, mem_logger_read_line
from threading import Thread
import logging
import time


log = logging.getLogger(__name__)


class SimavrLogger(object):
    def __init__(self):
        '''
        '''
        use_mem_logger()
        Thread(target=self._log_reader).start()

    def __del__(self):
        self.terminate()

    def terminate(self):
        self._terminate_log_thread = True

    _terminate_log_thread = False

    def _log_reader(self):
        while not self._terminate_log_thread:
            s = mem_logger_read_line()
            if s:
                self.log(s)
            else:
                time.sleep(0.01)

    def log(self, line):
        log.debug(line)


_simavr_logger = None


def init_simavr_logger():
    global _simavr_logger
    if not _simavr_logger:
        _simavr_logger = SimavrLogger()


def terminate_simavr_logger():
    global _simavr_logger
    if _simavr_logger:
        _simavr_logger.terminate()



from proxy import Proxy
from swig.inverter import inverter_t, inverter_core_init, IRQ_INVERTER_IN, \
    IRQ_INVERTER_OUT
from swig.simavr import get_irq_at
import time


class Inverter(Proxy):
    _reserved = 'getirq pinstate'.split()
    _pins = dict(
        IN=IRQ_INVERTER_IN,
        OUT=IRQ_INVERTER_OUT,
    )

    def __init__(self, avr):
        self.backend = inverter_t()
        inverter_core_init(avr.backend, self.backend)

    def _getirq(self, index):
        return get_irq_at(self.backend.irq, index)

    def getirq(self, pin):
        return self._getirq(self._pins[pin])

    def out(self, i):
        return bool(self.backend.out & (1 << i))



from pysimavr.swig.uart_buff import uart_buff_t, uart_buff_init, \
    uart_buff_connect, uart_buff_terminate, read_fifo, write_fifo
from threading import Thread
import logging
import string
import time

log = logging.getLogger(__name__)


class Uart():
    _terminate_log_thread = False

    def __init__(self, avr):
        self.buffer = []
        self.line = ''
        self.backend = uart_buff_t()
        uart_buff_init(avr.backend, self.backend)
        Thread(target=self._uart_reader).start()

    def connect(self):
        uart_buff_connect(self.backend, '0')

    def terminate(self):
        uart_buff_terminate(self.backend)
        self._terminate_log_thread = True

    def __del__(self):
        self.terminate()

    def _uart_reader(self):
        while not self._terminate_log_thread:
            x = read_fifo(self.backend.fifo_in)
            if x == -1:
                time.sleep(0.01)
            else:
                self.uart_log(chr(x))

    def uart_log(self, c):
        self.buffer.append(c)

        if c == '\n':
            log.debug(self.line)
            self.line = ''
        else:
            if c not in string.printable:
                x = '.'
            else:
                x = c
            self.line += x

    def send_string(self, s):
        for c in s:
            x = write_fifo(self.backend.fifo_out, ord(c))



from proxy import Proxy
from pysimavr.swig.spk import spk_reset, spk_t, spk_core_init, spk_read, \
    spk_buffer_ready
from swig.simavr import get_irq_at
import time


class Spk(Proxy):
    _reserved = 'speed rate read getirq'.split()

    def __init__(self, avr, rate=11025, speed=1.0):
        self.rate = rate
        self.speed = speed
        self.backend = spk_t()
        spk_core_init(avr.backend, self.backend, rate, speed)

    def getirq(self, pin):
        return get_irq_at(self.backend.irq, 0)

    def read(self):
#        assert not self.backend.overrun
        i = 0
        while not spk_buffer_ready(self.backend):
            i += 1
            time.sleep(0.005)
            if i > 1000:
                print '----------'
                break
        buffer = spk_read(self.backend)
#        print 'buffer ', len(buffer), buffer
        spk_reset(self.backend)
#        print 'reseet ', self.backend.start_cycle
        return buffer



from swig.simavr import avr_connect_irq
import logging
import operator

log = logging.getLogger(__name__)


def connect_irqs(irq_out, irq_in, bidirectional=False):
    avr_connect_irq(irq_out, irq_in)
    if bidirectional:
        avr_connect_irq(irq_in, irq_out)


def connect_pins_by_rule(rule, device_map, vcd=None):
    ''' rule example:

    B0 --> D4 -> vcd

    B1 <== D5
    B2  => D6
    # B3 <=> D7
    '''
    lines = rule.strip().splitlines()

    # remove comments
    lines = [x.split('#')[0] for x in lines]

    # remove empty lines
    lines = [x.strip() for x in lines]
    lines = filter(bool, lines)

    # split by '|'
    if len(lines):
        lines = reduce(operator.add, [x.split('|') for x in lines])

    lines = [x.strip().replace(
        '-', '=').replace('==', '=').replace(' ', '') for x in lines]

    class _dev(object):
        def __init__(self):
            self.vcd = None
            self.dev = None
            self.devname = None
            self.irq = None

    def addsignal(vcd, d):
        name = '%s.%s' % (d.devname, d.irqname)
        vcd.add_signal(d.irq, name=name)

    def con(d1, d2):
        if d1.vcd and d2.dev:
            addsignal(d1.vcd, d2)
        elif d2.vcd and d1.dev:
            addsignal(d2.vcd, d1)
        elif d1.dev and d2.dev:
            connect_irqs(d1.irq, d2.irq, bidirectional=False)
            log.debug('connecting %s -> %s' %
                     ((d1.devname, d1.irqname), (d2.devname, d2.irqname)))

    def getdev(s):
        d = _dev()
        if s == 'vcd':
            d.vcd = vcd
        else:
            ls = s.split('.')
            d.devname = ls[0]
            d.irqname = ls[1]
            d.dev = device_map[ls[0]]
            if d.dev:
                d.irq = d.dev.getirq(ls[1])
        return d

    def process(x):
        if '<=>' in x:
            toleft = 1
            toright = 1
        elif '<=' in x:
            toleft = 1
            toright = 0
        elif '=>' in x:
            toleft = 0
            toright = 1
        else:
            toleft = 1
            toright = 1
        x = x.replace('>', '')
        x = x.replace('<', '')
        ls = map(getdev, x.split('='))

        if toright:
            con(ls[0], ls[1])
        if toleft:
            con(ls[1], ls[0])

    for x in lines:
        elems = x.split('=')
        pairs = zip(elems[:-1], elems[1:])
        for a, b in pairs:
            process(a + '=' + b)

# def connect_pins_by_map(dev1, dev2, rule):
#    ''' rule example:
#
#    B0 --> D4
#
#    B1 <== D5
#    B2  => D6
#    #B3 <=> D7
#    '''
#    lines = [x.strip() for x in rule.strip().splitlines()]
#    lines = filter(bool, lines)
#    lines = filter(lambda x: '#' not in x, lines)
#    def con(irq1, irq2):
#        connect_irqs(irq1, irq2, bidirectional=False)
#    for x in lines:
#        x = x.strip().replace('-', '=').replace('==', '=').replace(' ', '')
#        if '<=>' in x:
#            toleft = 1
#            toright = 1
#        elif '<=' in x:
#            toleft = 1
#            toright = 0
#        elif '=>' in x:
#            toleft = 0
#            toright = 1
#        else:
#            toleft = 1
#            toright = 1
#        x = x.replace('>', '')
#        x = x.replace('<', '')
#        ls = x.split('=')
#        irq1 = dev1.getirq(ls[0])
#        irq2 = dev2.getirq(ls[1])
#        if toright:
#            log.debug('connecting %s -> %s' % tuple(ls))
#            con(irq1, irq2)
#        if toleft:
#            log.debug('connecting %s -> %s' % tuple(reversed(ls)))
#            con(irq2, irq1)



from proxy import Proxy
from swig.simavr import avr_vcd_t, avr_vcd_start, avr_vcd_stop, avr_vcd_init, \
    avr_vcd_add_signal, avr_vcd_close
import logging

log = logging.getLogger(__name__)


class VcdFile(Proxy):
    _reserved = 'terminate start stop add_signal'.split()

    def __init__(self, avr, filename="gtkwave_output.vcd", period=10):
        '''period : usec

        if period is too high:
        _avr_vcd_notify lcd.D4 overrun value buffer 256
        '''
        self.backend = avr_vcd_t()
#        print avr, filename, self.backend, period
        avr_vcd_init(avr.backend, filename, self.backend, period)

    def start(self):
        avr_vcd_start(self.backend)

    def stop(self):
        avr_vcd_stop(self.backend)

    def add_signal(self, irq, name=None, bits=1):
        if not name:
            name = irq.name
        log.debug('vcd.add_signal: %s' % name)
        avr_vcd_add_signal(self.backend, irq, bits, name)

    _terminated=False
    def terminate(self):
        if self._terminated:
            return
        self._terminated=True
        log.debug('terminating...')
        avr_vcd_close(self.backend)
        log.debug('...ok')

    def __del__(self):
        self.terminate()





class Proxy(object):
    _reserved = []

    def __init__(self):
        pass

    def __getattribute__(self, name):
        if name == 'backend' or name.startswith('_') or name in self._reserved:
            return object.__getattribute__(self, name)
        else:
            return getattr(object.__getattribute__(self, 'backend'), name)
#    def __delattr__(self, name):
#        delattr(object.__getattribute__(self, "_obj"), name)

    def __setattr__(self, name, value):
        if name == 'backend' or name.startswith('_') or name in self._reserved:
            object.__setattr__(self, name, value)
        else:
            setattr(object.__getattribute__(self, 'backend'), name, value)



from proxy import Proxy
from swig.simavr import get_irq_at

from swig.ac_input import IRQ_AC_OUT, ac_input_t, ac_input_init


class Ac(Proxy):
    _reserved = 'getirq'.split()

    def __init__(self, avr):
        self.backend = ac_input_t()
        ac_input_init(avr.backend, self.backend)

    def _getirq(self, index):
        return get_irq_at(self.backend.irq, index)

    def getirq(self, pin):
        return self._getirq(IRQ_AC_OUT)



from swig.uart_udp import uart_udp_init, uart_udp_connect, uart_udp_t, uart_udp_terminate
import logging

log = logging.getLogger(__name__)


class Udp():
    def __init__(self, avr):
        self.backend = uart_udp_t()
        uart_udp_init(avr.backend, self.backend)

    def connect(self):
        uart_udp_connect(self.backend, '0')

    def terminate(self):
        uart_udp_terminate(self.backend)

    def __del__(self):
        self.terminate()



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 2.0.4
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.



from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_hd44780', [dirname(__file__)])
        except ImportError:
            import _hd44780
            return _hd44780
        if fp is not None:
            try:
                _mod = imp.load_module('_hd44780', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _hd44780 = swig_import_helper()
    del swig_import_helper
else:
    import _hd44780
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0


IRQ_HD44780_ALL = _hd44780.IRQ_HD44780_ALL
IRQ_HD44780_RS = _hd44780.IRQ_HD44780_RS
IRQ_HD44780_RW = _hd44780.IRQ_HD44780_RW
IRQ_HD44780_E = _hd44780.IRQ_HD44780_E
IRQ_HD44780_D0 = _hd44780.IRQ_HD44780_D0
IRQ_HD44780_D1 = _hd44780.IRQ_HD44780_D1
IRQ_HD44780_D2 = _hd44780.IRQ_HD44780_D2
IRQ_HD44780_D3 = _hd44780.IRQ_HD44780_D3
IRQ_HD44780_D4 = _hd44780.IRQ_HD44780_D4
IRQ_HD44780_D5 = _hd44780.IRQ_HD44780_D5
IRQ_HD44780_D6 = _hd44780.IRQ_HD44780_D6
IRQ_HD44780_D7 = _hd44780.IRQ_HD44780_D7
IRQ_HD44780_INPUT_COUNT = _hd44780.IRQ_HD44780_INPUT_COUNT
IRQ_HD44780_BUSY = _hd44780.IRQ_HD44780_BUSY
IRQ_HD44780_ADDR = _hd44780.IRQ_HD44780_ADDR
IRQ_HD44780_DATA_IN = _hd44780.IRQ_HD44780_DATA_IN
IRQ_HD44780_DATA_OUT = _hd44780.IRQ_HD44780_DATA_OUT
IRQ_HD44780_COUNT = _hd44780.IRQ_HD44780_COUNT
HD44780_FLAG_F = _hd44780.HD44780_FLAG_F
HD44780_FLAG_N = _hd44780.HD44780_FLAG_N
HD44780_FLAG_D_L = _hd44780.HD44780_FLAG_D_L
HD44780_FLAG_R_L = _hd44780.HD44780_FLAG_R_L
HD44780_FLAG_S_C = _hd44780.HD44780_FLAG_S_C
HD44780_FLAG_B = _hd44780.HD44780_FLAG_B
HD44780_FLAG_C = _hd44780.HD44780_FLAG_C
HD44780_FLAG_D = _hd44780.HD44780_FLAG_D
HD44780_FLAG_S = _hd44780.HD44780_FLAG_S
HD44780_FLAG_I_D = _hd44780.HD44780_FLAG_I_D
HD44780_FLAG_LOWNIBBLE = _hd44780.HD44780_FLAG_LOWNIBBLE
HD44780_FLAG_BUSY = _hd44780.HD44780_FLAG_BUSY
HD44780_FLAG_REENTRANT = _hd44780.HD44780_FLAG_REENTRANT
HD44780_FLAG_DIRTY = _hd44780.HD44780_FLAG_DIRTY
HD44780_FLAG_CRAM_DIRTY = _hd44780.HD44780_FLAG_CRAM_DIRTY
VRAM_SIZE = _hd44780.VRAM_SIZE
class hd44780_t(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, hd44780_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, hd44780_t, name)
    __repr__ = _swig_repr
    __swig_setmethods__["irq"] = _hd44780.hd44780_t_irq_set
    __swig_getmethods__["irq"] = _hd44780.hd44780_t_irq_get
    if _newclass:irq = _swig_property(_hd44780.hd44780_t_irq_get, _hd44780.hd44780_t_irq_set)
    __swig_setmethods__["avr"] = _hd44780.hd44780_t_avr_set
    __swig_getmethods__["avr"] = _hd44780.hd44780_t_avr_get
    if _newclass:avr = _swig_property(_hd44780.hd44780_t_avr_get, _hd44780.hd44780_t_avr_set)
    __swig_setmethods__["w"] = _hd44780.hd44780_t_w_set
    __swig_getmethods__["w"] = _hd44780.hd44780_t_w_get
    if _newclass:w = _swig_property(_hd44780.hd44780_t_w_get, _hd44780.hd44780_t_w_set)
    __swig_setmethods__["h"] = _hd44780.hd44780_t_h_set
    __swig_getmethods__["h"] = _hd44780.hd44780_t_h_get
    if _newclass:h = _swig_property(_hd44780.hd44780_t_h_get, _hd44780.hd44780_t_h_set)
    __swig_setmethods__["cursor"] = _hd44780.hd44780_t_cursor_set
    __swig_getmethods__["cursor"] = _hd44780.hd44780_t_cursor_get
    if _newclass:cursor = _swig_property(_hd44780.hd44780_t_cursor_get, _hd44780.hd44780_t_cursor_set)
    __swig_setmethods__["vram"] = _hd44780.hd44780_t_vram_set
    __swig_getmethods__["vram"] = _hd44780.hd44780_t_vram_get
    if _newclass:vram = _swig_property(_hd44780.hd44780_t_vram_get, _hd44780.hd44780_t_vram_set)
    __swig_setmethods__["pinstate"] = _hd44780.hd44780_t_pinstate_set
    __swig_getmethods__["pinstate"] = _hd44780.hd44780_t_pinstate_get
    if _newclass:pinstate = _swig_property(_hd44780.hd44780_t_pinstate_get, _hd44780.hd44780_t_pinstate_set)
    __swig_setmethods__["datapins"] = _hd44780.hd44780_t_datapins_set
    __swig_getmethods__["datapins"] = _hd44780.hd44780_t_datapins_get
    if _newclass:datapins = _swig_property(_hd44780.hd44780_t_datapins_get, _hd44780.hd44780_t_datapins_set)
    __swig_setmethods__["readpins"] = _hd44780.hd44780_t_readpins_set
    __swig_getmethods__["readpins"] = _hd44780.hd44780_t_readpins_get
    if _newclass:readpins = _swig_property(_hd44780.hd44780_t_readpins_get, _hd44780.hd44780_t_readpins_set)
    __swig_setmethods__["flags"] = _hd44780.hd44780_t_flags_set
    __swig_getmethods__["flags"] = _hd44780.hd44780_t_flags_get
    if _newclass:flags = _swig_property(_hd44780.hd44780_t_flags_get, _hd44780.hd44780_t_flags_set)
    def __init__(self): 
        this = _hd44780.new_hd44780_t()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _hd44780.delete_hd44780_t
    __del__ = lambda self : None;
hd44780_t_swigregister = _hd44780.hd44780_t_swigregister
hd44780_t_swigregister(hd44780_t)


def hd44780_init(*args):
  return _hd44780.hd44780_init(*args)
hd44780_init = _hd44780.hd44780_init

def hd44780_print(*args):
  return _hd44780.hd44780_print(*args)
hd44780_print = _hd44780.hd44780_print

def hd44780_set_flag(*args):
  return _hd44780.hd44780_set_flag(*args)
hd44780_set_flag = _hd44780.hd44780_set_flag

def hd44780_get_flag(*args):
  return _hd44780.hd44780_get_flag(*args)
hd44780_get_flag = _hd44780.hd44780_get_flag

def hd44780_get_char(*args):
  return _hd44780.hd44780_get_char(*args)
hd44780_get_char = _hd44780.hd44780_get_char

def hd44780_reset(*args):
  return _hd44780.hd44780_reset(*args)
hd44780_reset = _hd44780.hd44780_reset
# This file is compatible with both classic and new-style classes.





from pyavrutils.arduino import Arduino
from pysimavr.avr import Avr
from pysimavr.connect import connect_pins_by_rule
from pysimavr.firmware import Firmware
from pysimavr.vcdfile import VcdFile
import logging
import time

log = logging.getLogger(__name__)

TEMPLATE = '''
void setup()
{
    Serial.begin(9600);

    snippet;

}

void loop()
{
}
'''


class ArduinoSim(object):
    '''arduino code builder and simulator for serial testing'''
    def __init__(self,
                 snippet=None,
                 mcu='atmega328p',
                 f_cpu=16000000,
                 extra_lib=None,
                 timespan=0.1,
                 vcd=None,
                 template=None,
                 code=None,
                 serial_in=None,
                 ):
        self.cc = Arduino(mcu=mcu, f_cpu=f_cpu, extra_lib=extra_lib)
        if template:
            self.template = template
        else:
            self.template = TEMPLATE
        self.snippet = snippet
        self.code = code
        self.timespan = timespan  # 10ms
        self.vcd = vcd
        self.serial = ''
        self.serial_in = serial_in

    @property
    def mcu(self):
        return self.cc.mcu

    @mcu.setter
    def mcu(self, value):
        self.cc.mcu = value

    def build(self):
        code = self.code
        if not code:
            code = self.template.replace('snippet', self.snippet)
        log.debug('code=%s' % code)
        self.cc.build(code)

    def simulate(self):
        elf = self.cc.output

        # run
        firmware = Firmware(elf)
        avr = Avr(mcu=self.cc.mcu, f_cpu=self.cc.f_cpu)
        avr.load_firmware(firmware)
#        udpReader = UdpReader()
#        udp = Udp(avr)
#        udp.connect()
#        udpReader.start()

        simvcd = None
        if self.vcd:
            simvcd = VcdFile(avr, period=1000, filename=self.vcd)
            connect_pins_by_rule('''
                                avr.D0 ==> vcd
                                avr.D1 ==> vcd
                                avr.D2 ==> vcd
                                avr.D3 ==> vcd
                                avr.D4 ==> vcd
                                avr.D5 ==> vcd
                                avr.D6 ==> vcd
                                avr.D7 ==> vcd

                                avr.B0 ==> vcd
                                avr.B1 ==> vcd
                                avr.B2 ==> vcd
                                avr.B3 ==> vcd
                                avr.B4 ==> vcd
                                avr.B5 ==> vcd
                                ''',
                                 dict(
                                 avr=avr,
                                 ),
                                 vcd=simvcd,
                                 )
            simvcd.start()

# not working
#        if self.serial_in:
#            avr.uart.send_string(self.serial_in)
        avr.move_time_marker(self.timespan)

        while avr.time_passed() < self.timespan * 0.99:
            time.sleep(0.05)

        if simvcd:
            simvcd.terminate()
#        udpReader.terminate()

        log.debug('cycles=%s' % avr.cycle)
        log.debug('mcu time=%s' % avr.time_passed())
#        time.sleep(1)
        self.serial = avr.uart.buffer
        avr.terminate()

    def run(self):
        self.build()
        self.simulate()

    def get_serial(self):
        self.run()
        return ''.join(self.serial)

    def size(self):
        self.build()
        return self.cc.size()

# def targets():
#    return Avr.arduino_targets
#
#
#
# def code2size(snippet, mcu):
#    return ArduinoSim(snippet=snippet, mcu=mcu).size()
#
# def code2ser(snippet, mcu):
#    return ArduinoSim(snippet=snippet, mcu=mcu).get_serial()



from setuptools import find_packages, setup
from setuptools.extension import Extension
import fnmatch
import os.path
import sys


def read_project_version(package):
    py = os.path.join(package, '__init__.py')
    __version__ = None
    for line in open(py).read().splitlines():
        if '__version__' in line:
            exec(line)
            break
    return __version__

NAME = 'pysimavr'
URL = 'https://github.com/ponty/pysimavr'
DESCRIPTION = 'python wrapper for simavr which is AVR and arduino simulator.'
VERSION = read_project_version(NAME)

extra = {}
if sys.version_info >= (3,):
    extra['use_2to3'] = True

classifiers = [
    # Get more strings from
    # http://www.python.org/pypi?%3Aaction=list_classifiers
    "License :: OSI Approved :: BSD License",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
]

install_requires = open("requirements.txt").read().split('\n')


def listdir(directory, pattern):
    names = os.listdir(directory)
    names = fnmatch.filter(names, pattern)
    return [os.path.join(directory, child) for child in names]


def files(directory, pattern):
    return [p for p in listdir(directory, pattern) if os.path.isfile(p)]


def part(name):
    return Extension(name='pysimavr.swig._' + name,
                     sources=[
                     'pysimavr/swig/parts/' + name + '.c',
                     'pysimavr/swig/' + name + '.i',
#                     'pysimavr/swig/sim/sim_cycle_timers.c',
#                     'pysimavr/swig/sim/sim_irq.c',
#                     'pysimavr/swig/sim/sim_io.c',
                     ],
                     libraries=['elf'],
                     include_dirs=[
                     'pysimavr/swig/sim',
                     'pysimavr/swig/include',
                     'pysimavr/swig/parts',
                     ],
                     swig_opts=[
                     #                       '-modern',
                     '-Ipysimavr/swig/parts',
                     '-Ipysimavr/swig/sim',
                     ],
                     extra_compile_args=[
                     '--std=gnu99',
                     ],
                     )

ext_modules = [
    Extension(name='pysimavr.swig._simavr',
              sources=[
              'pysimavr/swig/simavr.i',
              'pysimavr/swig/simavr_logger.cpp',
              ]
              + files('pysimavr/swig/sim', '*.c')
              + files('pysimavr/swig/cores', 'sim_*.c'),
              libraries=['elf'],
              include_dirs=[
              'pysimavr/swig/sim',
              'pysimavr/swig/include',
              ],
              swig_opts=[
              #                       '-modern',
              '-Ipysimavr/swig/sim',
              ],
              extra_compile_args=[
              '--std=gnu99',
              '-DNO_COLOR',
              ],
              ),
    part('sgm7'),
    part('ledrow'),
    part('inverter'),
    part('hd44780'),
    part('ac_input'),
    part('button'),
    part('uart_udp'),
    part('spk'),
    part('uart_buff'),
    
]

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=open('README.rst', 'r').read(),
    classifiers=classifiers,
    keywords='avr simavr',
    author='ponty',
    # author_email='',
    url=URL,
    license='GPL',
    packages=find_packages(exclude=['bootstrap', 'pavement', ]),
    include_package_data=True,
    test_suite='nose.collector',
    zip_safe=False,
    install_requires=install_requires,
    ext_modules=ext_modules,
    **extra
)



from proxy import Proxy
from swig.sgm7 import sgm7_t, sgm7_core_init, sgm7_get_digit_segments, sgm7_reset_dirty
from swig.simavr import get_irq_at


class Sgm7(Proxy):
    _reserved = 'getirq pinstate reset_dirty pins pinindex digit_segments'.split()

    def __init__(self, avr, size=4):
        self.pins = 'A B C D E F G P'.split(
        ) + ['D' + str(x) for x in range(size)]
        self.backend = sgm7_t()
        sgm7_core_init(avr.backend, self.backend, size)

    def _getirq(self, index):
        return get_irq_at(self.backend.irq, index)

    def pinindex(self, pin_name):
        return self.pins.index(pin_name)

    def getirq(self, pin):
        return self._getirq(self.pinindex(pin))

    def pinstate(self, pin):
        return bool(self.backend.pinstate & (1 << self.pinindex(pin)))

    def digit_segments(self, digit_index):
        return sgm7_get_digit_segments(self.backend, digit_index)

    def reset_dirty(self, digit_index):
        'read and reset'
        return sgm7_reset_dirty(self.backend, digit_index)
